#!/usr/bin/env python
import pygame
from pygame.locals import *
import os
import sys

from setup import *
import cursor, tools, player, enemy, bar, shot, title, explosion, item

class Main():
    def __init__(self):
        pygame.init()
        pygame.display.set_caption(CAPTION)
        screen = pygame.display.set_mode(SCR_RECT.size)

        self.all = pygame.sprite.RenderUpdates()
        self.enemies = pygame.sprite.Group()
        self.shots = pygame.sprite.Group()
        # itemの衝突判定用グループを作成
        self.items = pygame.sprite.Group()
        player.Player.containers = self.all
        enemy.Enemy.containers = self.all, self.enemies
        shot.Shot.containers = self.all, self.shots
        explosion.Explosion.containers = self.all
        # グループを登録
        item.Item.containers = self.all, self.items

        self.load_enemy()
        # アイテムをロード
        self.load_item()
        self.player = player.Player((288, 200))

        self.bg = tools.load_image("data", "bg.png")
        self.bgx = 0
        self.scroll_speed = 3

        self.hp = bar.HealthBar(32, 32, 100, 12)
        self.font = pygame.font.Font("data/minamoji04.ttf", 24)
        self.enemy_name = ""

        self.load_sounds()
        tools.play_bgm("bgm.ogg")

        self.title = title.Title()

        clock = pygame.time.Clock()
        global game_state
        game_state = TITLE
        while True:
            clock.tick(30)
            self.update()
            self.draw(screen)
            pygame.display.update()
            self.key_handler()

    def update(self):
        global game_state
        if game_state == TITLE:
            self.title.update()
        elif game_state == STAGE:
            self.bgx = (self.bgx - self.scroll_speed) % SCR_RECT.width
            self.all.update()
            self.collision_detection()
            self.hp.update()
            self.name_label = self.font.render(self.enemy_name, True, (255, 255, 255))

    def draw(self, screen):
        global game_state
        if game_state == TITLE:
            self.title.draw(screen)
        elif game_state == STAGE:
            screen.blit(self.bg, (self.bgx, 0))
            screen.blit(self.bg, (self.bgx - SCR_RECT.width, 0))
            self.all.draw(screen)
            self.hp.draw(screen)
            screen.blit(self.name_label, (500, 450))

    def key_handler(self):
        global game_state
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()

            if game_state == TITLE:
                if event.type == KEYUP and event.key == K_SPACE:
                    if self.title.select == 0:
                        game_state = STAGE
                    elif self.title.select == 1:
                        pygame.quit()
                        sys.exit()
                elif event.type == KEYDOWN and event.key == K_UP:
                    if self.title.select > 0:
                        self.title.select -= 1
                elif event.type == KEYDOWN and event.key == K_DOWN:
                    if self.title.select < 1:
                        self.title.select += 1

    def collision_detection(self):
        enemy_collided = pygame.sprite.spritecollide(self.player, self.enemies, False)
        if enemy_collided:
            if self.player.invincible == False:
                self.damage_sound.play()
                self.hp.hp -= 1
                self.player.invincible = True
        shot_collided = pygame.sprite.groupcollide(self.enemies, self.shots, False, True)
        for enemy in shot_collided.keys():
            explosion.Explosion(enemy.rect.center)
            enemy.hp -= 1
            self.enemy_name = enemy.name
            if enemy.hp == 0:
                enemy.kill()
        # アイテムとの衝突判定
        item_collided = pygame.sprite.spritecollide(self.player, self.items, True)
        if item_collided:
            self.heal_sound.play()
            self.hp.hp += 1

    def load_sounds(self):
        self.damage_sound = tools.load_sound("damage.wav")
        self.heal_sound = tools.load_sound("heal.mp3")

    def load_enemy(self):
        file = os.path.join("data", "enemy.dat")
        f = open(file, "r", encoding="utf-8")
        for line in f:
            line = line.rstrip()
            data = line.split(",")
            self.create_enemys(data)
        f.close()

    def create_enemys(self, data):
        self.time = int(data[0])
        self.image = data[1]
        self.pos = (int(data[2]), int(data[3]))
        self.move_type = int(data[4])
        self.hp = int(data[5])
        self.name = data[6]

        enemy.Enemy(self.time, self.image, self.pos, self.move_type, self.hp, self.name)
    
    def load_item(self):
        # アイテムをロードする
        file = os.path.join("data", "item.dat")
        f = open(file, "r", encoding="utf-8")
        for line in f:
            line = line.rstrip()
            data = line.split(",")
            item.Item(int(data[0]), int(data[1]))
        f.close()

if __name__ == "__main__":
    Main()
